package PureMessage::HTTPD::Services::Monitor;

use strict;
use warnings;
use MIME::Entity;
use base qw( PureMessage::HTTPD::Services::Application );
use PureMessage::Monitor::EventQueue;

# Setup; specify name of logfile to use and list of available run modes.
sub setup {
    my $self = shift;
    $self->set_logfile( 'service_monitor_log' );
    $self->run_modes(
        'list'      => 'list_items',
        'transfer'  => 'transfer_items',
        );
}

# RUN MODE "list"; list the items in the queue
sub list_items {
    my $self = shift;
    my $qname = $self->_find_queue_name();
    my $queue = PureMessage::Monitor::EventQueue->new( $qname );

    # list the items in the queue
    my @items = $queue->items();
    $self->log( "found " . scalar(@items) . " items in '$qname' queue" );

    # send the list back
    $self->header_props( -type => 'text/plain' );
    return join("\n",@items);
}

# RUN MODE "transfer"; transfer items out of the queue (get and then remove)
sub transfer_items {
    my $self = shift;
    my $qname = $self->_find_queue_name();
    my $queue = PureMessage::Monitor::EventQueue->new( $qname );
    my @qids  = $self->query()->param( 'id' );

    # get the items out of the queue, removing each one in turn
    my %items;
    foreach my $id (@qids) {
        eval {
            my $event = $queue->get_item( $id );
            if ($queue->remove_item( $id )) {
                $items{$id} = $event;
            }
        };
        if ($@) {
            $self->log( "unable to transfer queue id '$id'; $@" );
        }
    }
    $self->log( "transferred " . scalar(keys %items) . " items from '$qname' queue" );

    # serialize the items and send them back, encapsulated in multipart MIME
    $self->header_props( -type => 'text/plain' );
    my $entity = MIME::Entity->build( Type => 'multipart/mixed' );
    foreach my $id (keys %items) {
        $entity->attach( 'Filename' => $id,
                         'Data' => $items{$id}->as_string(),
                       );
    }
    return $entity->as_string();
}

# Finds the name of the queue within the PATH_INFO.
#
# FYI, we know that we -always- have PATH_INFO, as otherwise we wouldn't have
# been invoked via the service dispatcher.
sub _find_queue_name {
    my $self = shift;
    my @path_info = split( /\//, $self->query()->path_info() );
    return $path_info[-1];
}

1;

=head1 NAME

PureMessage::HTTPD::Services::Monitor - HTTPD Service for PureMessage Monitor

=head1 SYNOPSIS

  # list items in monitor event queue
  http://<server>:28080/services/monitor/list/<event_queue>

  # transfer items out of monitor event queue
  # HTTP "POST" "id=<queue_id>" to this URL (supports multiple IDs)
  http://<server>:28080/services/monitor/transfer/<event_queue>

=head1 DESCRIPTION

C<PureMessage::HTTPD::Services::Monitor> implements an HTTPd service for the
PureMessage Monitor system.  Services are implemented to allow for the listing
of items within a given queue and to transfer items out of a queue.

=head1 SERVICES

=over

=item B</list/E<lt>event_queueE<gt>>

Lists the Queue IDs for all of the events in the named C<event_queue>.

Response is returned as plain text, with one ID per line.

=item B</transfer/E<lt>event_queueE<gt>>

Transfers one or more events from the named C<event_queue> into the HTTP
response.  Once transferred, the items are automatically removed from the event
queue.

To specify which events to transfer, HTTP POST their Queue IDs (from the
C</list/...> service) to this service as C<id=...>.  Multiple events can be
transferred in a single HTTP request.

Response is returned as plain text, using MIME encapsulation for all events
transferred (even in the case of a single event being transferred).

=back

=head1 COPYRIGHT

Copyright (C) 2000-2006 Sophos Group.  All rights reserved.  Sophos and
PureMessage are trademarks of Sophos Plc and Sophos Group.

=cut
